@@ -8,11 +8,14 @@ __email__ = "arenas.guerrero.julian@outlook.com"
 
 import json
 import pandas as pd
+import numpy as np
+import elementpath
+import xml.etree.ElementTree as ET
 
-from lxml import etree
 from jsonpath import JSONPath
 
 from ..constants import *
+from ..utils import normalize_hierarchical_data
 
 
 def get_file_data(config, mapping_rule, references):
@@ -129,32 +132,45 @@ def _read_json(mapping_rule, references):
         json_data = json.load(json_file)
 
     jsonpath_expression = mapping_rule['iterator'] + '.('
+    # add top level object of the references to reduce intermediate results (THIS IS NOT STRICTLY NECESSARY)
     for reference in references:
-        jsonpath_expression += reference + ','
+        jsonpath_expression += reference.split('.')[0] + ','
     jsonpath_expression = jsonpath_expression[:-1] + ')'
 
     jsonpath_result = JSONPath(jsonpath_expression).parse(json_data)
-    json_df = pd.DataFrame.from_records(jsonpath_result)
-    json_df.dropna(axis=0, how='any', inplace=True)
+    json_df = pd.json_normalize([json_object for json_object in normalize_hierarchical_data(jsonpath_result)])
 
-    # in case lists where retrieved, explode them e.g.: ['a', 'b'] -> 'a', 'b'
-    for reference in references:
-        json_df = json_df.explode(reference)
+    # add columns with null values for those references in the mapping rule that are not present in the data file
+    missing_references_in_df = list(set(references).difference(set(json_df.columns)))
+    json_df[missing_references_in_df] = np.nan
+
+    # keep only reference columns in the dataframe and remove NULLs
+    json_df = json_df[references]
+    json_df.dropna(axis=0, how='any', inplace=True)
 
     return [json_df]
 
 
 def _read_xml(mapping_rule, references):
     with open(str(mapping_rule['data_source']), encoding='utf-8') as xml_file:
-        xml_root = etree.parse(xml_file).getroot()
+        xml_root = ET.parse(xml_file).getroot()
 
-    xpath_expression = mapping_rule['iterator']
-    for reference in references:
-        xpath_expression += '[' + reference + ']'
+    xpath_result = elementpath.iter_select(xml_root, mapping_rule['iterator'])  # XPath2Parser by default
+    xpath_result = [[[r.text for r in e.findall(reference)] for reference in references] for e in xpath_result]
 
-    xpath_result = xml_root.xpath(xpath_expression)
-    xpath_result = [[e.find(reference).text for reference in references] for e in xpath_result]
+    # IMPORTANT NOTES
+    # XPath 2.0 is used by default (XPath 3.1 is in the roadmap of the elementpath library)
+    # with XPath 3.1 the above could be achieved using just an XPath expression by including the references in it
+    # for instance, the XPath expression: /root/[id,creator/name] obtaining for example ["2479", ["Julián", "Jhon"]]
 
     xml_df = pd.DataFrame.from_records(xpath_result, columns=references)
 
+    # add columns with null values for those references in the mapping rule that are not present in the data file
+    missing_references_in_df = list(set(references).difference(set(xml_df.columns)))
+    xml_df[missing_references_in_df] = np.nan
+    xml_df.dropna(axis=0, how='any', inplace=True)
+
+    for reference in references:
+        xml_df = xml_df.explode(reference)
+
     return [xml_df]
